<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Export\ExportContentToGitStore;

use App\Context\Content\Application\Process\Export\Content\ExportContentService;
use App\Context\Content\Application\Service\Git\GitStore;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ExportContentToGitStoreHandler implements CommandHandler
{
    public function __construct(
        protected readonly GitStore $gitStore,
        protected readonly ExportContentService $exportContentService,
    ) {
    }

    public function __invoke(ExportContentToGitStoreCommand $command): ExportContentToGitStoreResponse
    {
        try {
            $this->gitStore->pull();
            $this->exportContentService->run();
            $this->gitStore->push();
        } finally {
            $this->gitStore->clean();
        }

        return new ExportContentToGitStoreResponse();
    }
}
