<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\ChangeLevelDescription;

use App\Context\Content\Domain\Model\Level\LevelDescription;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeLevelDescriptionHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelEditPolicy $levelEditPolicy,
    ) {
    }

    public function __invoke(ChangeLevelDescriptionCommand $command): ChangeLevelDescriptionCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));

        $description = new LevelDescription($command->description ?? '');
        $level->changeDescription($this->levelEditPolicy, $description);

        $this->persistence->persist($level);

        return new ChangeLevelDescriptionCommandResponse($level->getId());
    }
}
