<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\ChangeLevelOwner;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeLevelOwnerHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $reader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelEditPolicy $levelEditPolicy,
    ) {
    }

    public function __invoke(ChangeLevelOwnerCommand $command): ChangeLevelOwnerCommandResponse
    {
        $level = $this->reader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));

        $level->changeOwner($this->levelEditPolicy, new MemberId($command->memberId));

        $this->persistence->persist($level);

        return new ChangeLevelOwnerCommandResponse($level->getId());
    }
}
