<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\ChangeLevelSlug;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\LevelSlug;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Level\Specification\LevelSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeLevelSlugHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelSlugUniqueSpec $levelSlugUniqueSpec,
        private readonly LevelEditPolicy $levelEditPolicy,
    ) {
    }

    public function __invoke(ChangeLevelSlugCommand $command): ChangeLevelSlugCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));

        $level->changeSlug($this->levelEditPolicy, $this->levelSlugUniqueSpec, new LevelSlug($command->slug));

        $this->persistence->persist($level);

        return new ChangeLevelSlugCommandResponse($level->getId());
    }
}
