<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\ChangeLevelTitle;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\LevelTitle;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeLevelTitleHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelEditPolicy $levelEditPolicy,
    ) {
    }

    public function __invoke(ChangeLevelTitleCommand $command): ChangeLevelTitleCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));

        $level->changeTitle($this->levelEditPolicy, new LevelTitle($command->title));

        $this->persistence->persist($level);

        return new ChangeLevelTitleCommandResponse($level->getId());
    }
}
