<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\CreateLevel;

use App\Context\Content\Domain\Model\Level\LevelDescription;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelSlug;
use App\Context\Content\Domain\Model\Level\LevelTitle;
use App\Context\Content\Domain\Model\Level\Policy\LevelCreatePolicy;
use App\Context\Content\Domain\Model\Level\Specification\LevelSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class CreateLevelHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelSlugUniqueSpec $levelSlugUniqueSpec,
        private readonly LevelCreatePolicy $levelCreatePolicy,
    ) {
    }

    public function __invoke(CreateLevelCommand $command): CreateLevelCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));
        $level = $skill->createLevel(
            $this->levelCreatePolicy,
            new LevelTitle($command->title),
            $this->levelSlugUniqueSpec,
            new LevelSlug($command->slug),
            null === $command->description ? null : new LevelDescription($command->description),
            $command->levelId ? new LevelId($command->levelId) : null,
        );

        $this->persistence->persist($level);

        return new CreateLevelCommandResponse($level->getId());
    }
}
