<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\RemoveLevel;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\Policy\LevelRemovePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RemoveLevelHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelRemovePolicy $levelRemovePolicy,
    ) {
    }

    public function __invoke(RemoveLevelCommand $command): RemoveLevelCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));
        $level->remove($this->levelRemovePolicy);

        $this->persistence->persist($level);

        return new RemoveLevelCommandResponse($level->getId());
    }
}
