<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\RestoreLevel;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\Policy\LevelRestorePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RestoreLevelHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelRestorePolicy $levelRestorePolicy,
    ) {
    }

    public function __invoke(RestoreLevelCommand $command): RestoreLevelCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));
        $level->restore($this->levelRestorePolicy);

        $this->persistence->persist($level);

        return new RestoreLevelCommandResponse($level->getId());
    }
}
