<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Level\UpdateLevel;

use App\Context\Content\Domain\Model\Level\Exception\DescriptionEqualsCurrentException;
use App\Context\Content\Domain\Model\Level\Exception\SlugEqualsCurrentException;
use App\Context\Content\Domain\Model\Level\Exception\TitleEqualsCurrentException;
use App\Context\Content\Domain\Model\Level\LevelDescription;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\LevelSlug;
use App\Context\Content\Domain\Model\Level\LevelTitle;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Level\Specification\LevelSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class UpdateLevelHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $repositoryReader,
        private readonly LevelRepositoryPersistence $persistence,
        private readonly LevelSlugUniqueSpec $levelSlugUniqueSpec,
        private readonly LevelEditPolicy $levelEditPolicy,
    ) {
    }

    public function __invoke(UpdateLevelCommand $command): UpdateLevelCommandResponse
    {
        $level = $this->repositoryReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));

        if (null !== $command->title) {
            try {
                $level->changeTitle($this->levelEditPolicy, new LevelTitle($command->title));
            } catch (TitleEqualsCurrentException) {
            }
        }

        if (null !== $command->slug) {
            try {
                $level->changeSlug(
                    $this->levelEditPolicy,
                    $this->levelSlugUniqueSpec,
                    new LevelSlug($command->slug)
                );
            } catch (SlugEqualsCurrentException) {
            }
        }

        if (null !== $command->description) {
            try {
                $level->changeDescription($this->levelEditPolicy, new LevelDescription($command->description));
            } catch (DescriptionEqualsCurrentException) {
            }
        }

        $this->persistence->persist($level);

        return new UpdateLevelCommandResponse($level->getId());
    }
}
