<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Member\RegisterMember;

use App\Context\Content\Domain\Model\Member\Member;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Content\Domain\Model\Member\MemberName;
use App\Context\Content\Domain\Model\Member\MemberRepositoryPersistence;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RegisterMemberHandler implements CommandHandler
{
    public function __construct(
        private readonly MemberRepositoryPersistence $persistence,
    ) {
    }

    public function __invoke(RegisterMemberCommand $command): RegisterMemberCommandResponse
    {
        $member = Member::create(
            new MemberId($command->id),
            new MemberName($command->name)
        );

        $this->persistence->persist($member);

        return new RegisterMemberCommandResponse($member->getId());
    }
}
