<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\ChangeQuestionDescription;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Policy\QuestionEditPolicy;
use App\Context\Content\Domain\Model\Question\QuestionDescription;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeQuestionDescriptionHandler implements CommandHandler
{
    public function __construct(
        private readonly QuestionRepositoryReader $questionReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionEditPolicy $questionEditPolicy,
    ) {
    }

    public function __invoke(ChangeQuestionDescriptionCommand $command): ChangeQuestionDescriptionCommandResponse
    {
        $skillId = new SkillId($command->skillId);
        $levelId = new LevelId($command->levelId);
        $questionId = new QuestionId($command->questionId);
        $question = $this->questionReader->getActiveById($skillId, $levelId, $questionId);

        $description = new QuestionDescription($command->description ?? '');
        $question->changeDescription($this->questionEditPolicy, $description);

        $this->persistence->persist($question);

        return new ChangeQuestionDescriptionCommandResponse($question->getId());
    }
}
