<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\ChangeQuestionSlug;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Policy\QuestionEditPolicy;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Question\QuestionSlug;
use App\Context\Content\Domain\Model\Question\Specification\QuestionSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeQuestionSlugHandler implements CommandHandler
{
    public function __construct(
        private readonly QuestionRepositoryReader $questionReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionSlugUniqueSpec $questionSlugUniqueSpec,
        private readonly QuestionEditPolicy $questionEditPolicy,
    ) {
    }

    public function __invoke(ChangeQuestionSlugCommand $command): ChangeQuestionSlugCommandResponse
    {
        $skillId = new SkillId($command->skillId);
        $levelId = new LevelId($command->levelId);
        $questionId = new QuestionId($command->questionId);
        $question = $this->questionReader->getActiveById($skillId, $levelId, $questionId);

        $question->changeSlug(
            $this->questionEditPolicy,
            $this->questionSlugUniqueSpec,
            new QuestionSlug($command->slug)
        );

        $this->persistence->persist($question);

        return new ChangeQuestionSlugCommandResponse($question->getId());
    }
}
