<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\ChangeQuestionTitle;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Policy\QuestionEditPolicy;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Question\QuestionTitle;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeQuestionTitleHandler implements CommandHandler
{
    public function __construct(
        private readonly QuestionRepositoryReader $questionReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionEditPolicy $questionEditPolicy,
    ) {
    }

    public function __invoke(ChangeQuestionTitleCommand $command): ChangeQuestionTitleCommandResponse
    {
        $skillId = new SkillId($command->skillId);
        $levelId = new LevelId($command->levelId);
        $questionId = new QuestionId($command->questionId);
        $question = $this->questionReader->getActiveById($skillId, $levelId, $questionId);

        $question->changeTitle($this->questionEditPolicy, new QuestionTitle($command->title));

        $this->persistence->persist($question);

        return new ChangeQuestionTitleCommandResponse($question->getId());
    }
}
