<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\CreateQuestion;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Question\Policy\QuestionCreatePolicy;
use App\Context\Content\Domain\Model\Question\QuestionDescription;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionSlug;
use App\Context\Content\Domain\Model\Question\QuestionTitle;
use App\Context\Content\Domain\Model\Question\Specification\QuestionSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class CreateQuestionHandler implements CommandHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionSlugUniqueSpec $questionSlugUniqueSpec,
        private readonly QuestionCreatePolicy $questionCreatePolicy,
    ) {
    }

    public function __invoke(CreateQuestionCommand $command): CreateQuestionCommandResponse
    {
        $level = $this->levelReader->getActiveById(new SkillId($command->skillId), new LevelId($command->levelId));
        $question = $level->createQuestion(
            $this->questionCreatePolicy,
            new QuestionTitle($command->title),
            $this->questionSlugUniqueSpec,
            new QuestionSlug($command->slug),
            null === $command->description ? null : new QuestionDescription($command->description),
            $command->questionId ? new QuestionId($command->questionId) : null
        );

        $this->persistence->persist($question);

        return new CreateQuestionCommandResponse($question->getId());
    }
}
