<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\RemoveQuestion;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Policy\QuestionRemovePolicy;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RemoveQuestionHandler implements CommandHandler
{
    public function __construct(
        private readonly QuestionRepositoryReader $questionReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionRemovePolicy $questionRemovePolicy,
    ) {
    }

    public function __invoke(RemoveQuestionCommand $command): RemoveQuestionCommandResponse
    {
        $skillId = new SkillId($command->skillId);
        $levelId = new LevelId($command->levelId);
        $questionId = new QuestionId($command->questionId);
        $question = $this->questionReader->getActiveById($skillId, $levelId, $questionId);
        $question->remove($this->questionRemovePolicy);

        $this->persistence->persist($question);

        return new RemoveQuestionCommandResponse($question->getId());
    }
}
