<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Question\RestoreQuestion;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Policy\QuestionRestorePolicy;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RestoreQuestionHandler implements CommandHandler
{
    public function __construct(
        private readonly QuestionRepositoryReader $questionReader,
        private readonly QuestionRepositoryPersistence $persistence,
        private readonly QuestionRestorePolicy $questionRestorePolicy,
    ) {
    }

    public function __invoke(RestoreQuestionCommand $command): RestoreQuestionCommandResponse
    {
        $skillId = new SkillId($command->skillId);
        $levelId = new LevelId($command->levelId);
        $questionId = new QuestionId($command->questionId);
        $question = $this->questionReader->getActiveById($skillId, $levelId, $questionId);
        $question->restore($this->questionRestorePolicy);

        $this->persistence->persist($question);

        return new RestoreQuestionCommandResponse($question->getId());
    }
}
