<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Command\Skill\AssignSkillModerator;

use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class AssignSkillModeratorHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillEditPolicy $skillEditPolicy,
    ) {
    }

    public function __invoke(AssignSkillModeratorCommand $command): AssignSkillModeratorCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));

        $skill->assignModerator($this->skillEditPolicy, new MemberId($command->memberId));

        $this->persistence->persist($skill);

        return new AssignSkillModeratorCommandResponse($skill->getId());
    }
}
