<?php

namespace App\Context\Content\Application\Command\Skill\ChangeSkillDescription;

use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillDescription;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeSkillDescriptionHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillEditPolicy $skillEditPolicy,
    ) {
    }

    public function __invoke(ChangeSkillDescriptionCommand $command): ChangeSkillDescriptionCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));

        $description = new SkillDescription($command->description ?? '');
        $skill->changeDescription($this->skillEditPolicy, $description);

        $this->persistence->persist($skill);

        return new ChangeSkillDescriptionCommandResponse($skill->getId());
    }
}
