<?php

namespace App\Context\Content\Application\Command\Skill\ChangeSkillSlug;

use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillSlug;
use App\Context\Content\Domain\Model\Skill\Specification\SkillSlugUniqueSpec;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeSkillSlugHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillSlugUniqueSpec $skillSlugUniqueSpec,
        private readonly SkillEditPolicy $skillEditPolicy,
    ) {
    }

    public function __invoke(ChangeSkillSlugCommand $command): ChangeSkillSlugCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));

        $skill->changeSlug($this->skillEditPolicy, $this->skillSlugUniqueSpec, new SkillSlug($command->slug));

        $this->persistence->persist($skill);

        return new ChangeSkillSlugCommandResponse($skill->getId());
    }
}
