<?php

namespace App\Context\Content\Application\Command\Skill\ChangeSkillTitle;

use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillTitle;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class ChangeSkillTitleHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillEditPolicy $skillEditPolicy,
    ) {
    }

    public function __invoke(ChangeSkillTitleCommand $command): ChangeSkillTitleCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));

        $skill->changeTitle($this->skillEditPolicy, new SkillTitle($command->title));

        $this->persistence->persist($skill);

        return new ChangeSkillTitleCommandResponse($skill->getId());
    }
}
