<?php

namespace App\Context\Content\Application\Command\Skill\CreateSkill;

use App\Context\Content\Domain\Model\Skill\Policy\SkillCreatePolicy;
use App\Context\Content\Domain\Model\Skill\Skill;
use App\Context\Content\Domain\Model\Skill\SkillDescription;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillSlug;
use App\Context\Content\Domain\Model\Skill\SkillTitle;
use App\Context\Content\Domain\Model\Skill\Specification\SkillSlugUniqueSpec;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class CreateSkillHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillSlugUniqueSpec $skillSlugUniqueSpec,
        private readonly SkillCreatePolicy $skillCreatePolicy,
    ) {
    }

    public function __invoke(CreateSkillCommand $command): CreateSkillCommandResponse
    {
        $skill = Skill::create(
            $this->skillCreatePolicy,
            new SkillTitle($command->title),
            $this->skillSlugUniqueSpec,
            new SkillSlug($command->slug),
            null === $command->description ? null : new SkillDescription($command->description),
            $command->id ? new SkillId($command->id) : null,
        );

        $this->persistence->persist($skill);

        return new CreateSkillCommandResponse($skill->getId());
    }
}
