<?php

namespace App\Context\Content\Application\Command\Skill\PatchSkill;

use App\Context\Content\Domain\Model\Skill\Exception\DescriptionEqualsCurrentException;
use App\Context\Content\Domain\Model\Skill\Exception\SlugEqualsCurrentException;
use App\Context\Content\Domain\Model\Skill\Exception\TitleEqualsCurrentException;
use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillDescription;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillSlug;
use App\Context\Content\Domain\Model\Skill\SkillTitle;
use App\Context\Content\Domain\Model\Skill\Specification\SkillSlugUniqueSpec;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class PatchSkillHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillEditPolicy $skillEditPolicy,
        private readonly SkillSlugUniqueSpec $skillSlugUniqueSpec,
    ) {
    }

    public function __invoke(PatchSkillCommand $command): PatchSkillCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));

        if (null !== $command->title) {
            try {
                $skill->changeTitle($this->skillEditPolicy, new SkillTitle($command->title));
            } catch (TitleEqualsCurrentException) {
            }
        }

        if (null !== $command->slug) {
            try {
                $skill->changeSlug(
                    $this->skillEditPolicy,
                    $this->skillSlugUniqueSpec,
                    new SkillSlug($command->slug)
                );
            } catch (SlugEqualsCurrentException) {
            }
        }

        if (null !== $command->description) {
            try {
                $skill->changeDescription($this->skillEditPolicy, new SkillDescription($command->description));
            } catch (DescriptionEqualsCurrentException) {
            }
        }

        $this->persistence->persist($skill);

        return new PatchSkillCommandResponse($skill->getId());
    }
}
