<?php

namespace App\Context\Content\Application\Command\Skill\RemoveSkill;

use App\Context\Content\Domain\Model\Skill\Policy\SkillRemovePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RemoveSkillHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillRemovePolicy $skillRemovePolicy,
    ) {
    }

    public function __invoke(RemoveSkillCommand $command): RemoveSkillCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));
        $skill->remove($this->skillRemovePolicy);

        $this->persistence->persist($skill);

        return new RemoveSkillCommandResponse($skill->getId());
    }
}
