<?php

namespace App\Context\Content\Application\Command\Skill\RestoreSkill;

use App\Context\Content\Domain\Model\Skill\Policy\SkillRestorePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Command\CommandHandler;

class RestoreSkillHandler implements CommandHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $reader,
        private readonly SkillRepositoryPersistence $persistence,
        private readonly SkillRestorePolicy $skillRestorePolicy,
    ) {
    }

    public function __invoke(RestoreSkillCommand $command): RestoreSkillCommandResponse
    {
        $skill = $this->reader->getActiveById(new SkillId($command->skillId));
        $skill->restore($this->skillRestorePolicy);

        $this->persistence->persist($skill);

        return new RestoreSkillCommandResponse($skill->getId());
    }
}
