<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Query\Level\Query\GetLevelList;

use App\Context\Content\Application\Query\Level\ReadModel\LevelReadRepository;
use App\Context\Content\Application\Query\Skill\ReadModel\SkillReadRepository;
use App\Context\Shared\Domain\Bus\Query\QueryHandler;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Domain\Criteria\Field;
use App\Context\Shared\Domain\Criteria\Filter;
use App\Context\Shared\Domain\Criteria\FilterOperator;
use App\Context\Shared\Domain\Criteria\FilterStringValue;

class GetLevelListQueryHandler implements QueryHandler
{
    public function __construct(
        private readonly SkillReadRepository $skillReadRepository,
        private readonly LevelReadRepository $levelReadRepository,
    ) {
    }

    public function __invoke(GetLevelListQuery $query): GetLevelListQueryResponse
    {
        $skill = $this->skillReadRepository->bySlug($query->skillSlug);

        $criteria = new Criteria();
        $criteria->filters()
            ->add(
                new Filter(
                    new Field('skill_id'),
                    FilterOperator::EQ,
                    new FilterStringValue($skill->id),
                ),
            );

        return new GetLevelListQueryResponse(
            $this->levelReadRepository->getList($criteria)
        );
    }
}
