<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Query\Level\Query\GetLevelPermissions;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy;
use App\Context\Content\Domain\Model\Level\Policy\LevelRemovePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Domain\Bus\Query\QueryHandler;

class GetLevelPermissionsQueryHandler implements QueryHandler
{
    public function __construct(
        private readonly LevelRepositoryReader $levelReadRepository,
        private readonly LevelEditPolicy $levelEditPolicy,
        private readonly LevelRemovePolicy $levelRemovePolicy,
    ) {
    }

    public function __invoke(GetLevelPermissionsQuery $query): GetLevelPermissionsQueryResponse
    {
        $level = $this->levelReadRepository->getActiveById(new SkillId($query->skillId), new LevelId($query->levelId));

        return new GetLevelPermissionsQueryResponse(
            $level->getId()->value(),
            $this->levelEditPolicy->can($level),
            $this->levelRemovePolicy->can($level),
        );
    }
}
