<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Query\Member\Query\GetMemberSearch;

use App\Context\Content\Application\Query\Member\ReadModel\MemberReadRepository;
use App\Context\Shared\Domain\Bus\Query\QueryHandler;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Domain\Criteria\Field;
use App\Context\Shared\Domain\Criteria\Filter;
use App\Context\Shared\Domain\Criteria\FilterOperator;
use App\Context\Shared\Domain\Criteria\FilterStringValue;

class GetMemberSearchQueryHandler implements QueryHandler
{
    public function __construct(
        private readonly MemberReadRepository $memberReadRepository,
    ) {
    }

    public function __invoke(GetMemberSearchQuery $query): GetMemberSearchQueryResponse
    {
        $criteria = new Criteria();
        $criteria->filters()
            ->add(
                new Filter(
                    new Field('name'),
                    FilterOperator::CONTAINS,
                    new FilterStringValue('%'.$query->q.'%'),
                ),
            );

        return new GetMemberSearchQueryResponse(
            $this->memberReadRepository->search($criteria)
        );
    }
}
