<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Query\Question\Query\GetQuestionList;

use App\Context\Content\Application\Query\Level\ReadModel\LevelReadRepository;
use App\Context\Content\Application\Query\Question\ReadModel\QuestionReadRepository;
use App\Context\Shared\Domain\Bus\Query\QueryHandler;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Domain\Criteria\Field;
use App\Context\Shared\Domain\Criteria\Filter;
use App\Context\Shared\Domain\Criteria\FilterOperator;
use App\Context\Shared\Domain\Criteria\FilterStringValue;

class GetQuestionListQueryHandler implements QueryHandler
{
    public function __construct(
        private readonly LevelReadRepository $levelReadRepository,
        private readonly QuestionReadRepository $questionReadRepository,
    ) {
    }

    public function __invoke(GetQuestionListQuery $query): GetQuestionListQueryResponse
    {
        $level = $this->levelReadRepository->bySlug($query->skillSlug, $query->levelSlug);

        $criteria = new Criteria();
        $criteria->filters()
            ->add(
                new Filter(
                    new Field('level_id'),
                    FilterOperator::EQ,
                    new FilterStringValue($level->id),
                ),
            );

        return new GetQuestionListQueryResponse(
            $this->questionReadRepository->getList($criteria)
        );
    }
}
