<?php

declare(strict_types=1);

namespace App\Context\Content\Application\Query\Skill\Query\GetSkillPermissions;

use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\Policy\SkillRemovePolicy;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Bus\Query\QueryHandler;

class GetSkillPermissionsQueryHandler implements QueryHandler
{
    public function __construct(
        private readonly SkillRepositoryReader $skillReadRepository,
        private readonly SkillEditPolicy $skillEditPolicy,
        private readonly SkillRemovePolicy $skillRemovePolicy,
    ) {
    }

    public function __invoke(GetSkillPermissionsQuery $query): GetSkillPermissionsQueryResponse
    {
        $skill = $this->skillReadRepository->getActiveById(new SkillId($query->skillId));

        return new GetSkillPermissionsQueryResponse(
            $skill->getId()->value(),
            $this->skillEditPolicy->can($skill),
            $this->skillRemovePolicy->can($skill),
        );
    }
}
