<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Export\Exporter;

use App\Context\Content\Application\Process\Export\Content\Dto\Skill;
use App\Context\Content\Application\Process\Export\Content\Exporter;
use App\Context\Content\Application\Process\Export\Content\ExportStore;
use Symfony\Component\Yaml\Yaml;

class YamlContentExporter implements Exporter
{
    public function export(Skill $skill, ExportStore $store): void
    {
        $path = sprintf('/skills/%s', $skill->slug);

        $rawSkill = [
            'id' => $skill->id,
            'slug' => $skill->slug,
            'title' => $this->clearTitle($skill->title),
            'description' => $skill->description,
            'sort' => $this->getSort($skill->title),
            'level_files' => [],
        ];

        foreach ($skill->levels as $level) {
            $rawLevel = [
                'id' => $level->id,
                'slug' => $level->slug,
                'title' => $this->clearTitle($level->title),
                'description' => $level->description,
                'questions' => [],
            ];

            foreach ($level->questions as $question) {
                $rawLevel['questions'][] = [
                    'id' => $question->id,
                    'slug' => $question->slug,
                    'title' => $this->clearTitle($question->title),
                    'description' => $question->description,
                ];
            }

            $relativeFilename = sprintf('levels/%s.yaml', $level->slug);
            $rawSkill['level_files'][] = sprintf('./%s', $relativeFilename);

            $filename = sprintf('%s/%s', $path, $relativeFilename);
            $store->store(
                $filename,
                Yaml::dump(
                    ['levels' => [$rawLevel]],
                    5,
                    2
                )
            );
        }

        $filename = sprintf('%s/%s.yaml', $path, $skill->slug);
        $store->store(
            $filename,
            Yaml::dump(
                [
                    'skills' => [$rawSkill],
                ],
                5,
                2
            )
        );
    }

    private function clearTitle(string $title): string
    {
        return trim(preg_replace('/^([0-9].\.)/', '', $title));
    }

    private function getSort(string $title): int
    {
        $matches = [];
        preg_match('/^([0-9]{2})/m', $title, $matches);
        if (isset($matches[0])) {
            return intval($matches[0]);
        }

        return 500;
    }
}
