<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Policy\Level;

use App\Context\Content\Domain\Model\Level\Policy\LevelEditPolicy as ILevelEditPolicy;
use App\Context\Content\Domain\Model\Skill\Policy\SkillEditPolicy;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Domain\Policy\Performer;

class LevelEditPolicy extends LevelPolicy implements ILevelEditPolicy
{
    public function __construct(
        Performer $performer,
        protected readonly SkillEditPolicy $policy,
        protected readonly SkillRepositoryReader $skillRepositoryReader
    ) {
        parent::__construct($performer);
    }

    public function can(object $object): bool
    {
        if (!parent::can($object)) {
            return false;
        }

        if ($object->assertIsOwner($this->memberId()) || $object->assertIsModerator($this->memberId())) {
            return true;
        }

        $skill = $this->skillRepositoryReader->findById($object->getSkillId());
        if (false === $this->policy->can($skill)) {
            return false;
        }

        return true;
    }
}
