<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Process\Export\Content\Dbal;

use App\Context\Content\Application\Process\Export\Content\Dto\Skill;
use App\Context\Content\Application\Process\Export\Content\LevelProvider;
use App\Context\Content\Application\Process\Export\Content\SkillProvider;
use Doctrine\DBAL\Connection;

class DbalSkillProvider implements SkillProvider
{
    public function __construct(
        private readonly Connection $connection,
        private readonly LevelProvider $levelProvider,
        private readonly int $batchSize = 100,
    ) {
    }

    public function get(): \Iterator
    {
        $offset = 0;
        do {
            $rawSkills = $this->connection->fetchAllAssociative(
                'SELECT id, slug, title, description'
                .' FROM skill s'
                .' WHERE s.is_removed = false '
                .' ORDER BY title'
                .' LIMIT ?'
                .' OFFSET ?',
                [
                    $this->batchSize,
                    $offset,
                ]
            );

            foreach ($rawSkills as $rawSkill) {
                yield $this->toObject($rawSkill);
            }

            $offset += $this->batchSize;
        } while ($rawSkills);
    }

    private function toObject(array $data): Skill
    {
        return new Skill(
            (string) ($data['id'] ?? ''),
            (string) ($data['slug'] ?? ''),
            (string) ($data['title'] ?? ''),
            $this->levelProvider->get([(string) $data['id']]),
            (string) ($data['description'] ?? '')
        );
    }
}
