<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Level\Doctrine;

use App\Context\Content\Application\Query\Level\ReadModel\LevelModeratorsReadRepository;
use App\Context\Content\Application\Query\Member\ReadModel\MemberReadModel;
use App\Context\Content\Domain\Model\Level\Exception\LevelNotFoundException;
use App\Context\Content\Domain\Model\Level\LevelId;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;

class LevelModeratorsReadModelRepositoryDoctrine implements LevelModeratorsReadRepository
{
    public function __construct(
        private readonly EntityManagerInterface $em,
    ) {
    }

    /**
     * @return list<MemberReadModel>
     *
     * @throws \Doctrine\DBAL\Exception
     *
     * @psalm-suppress MixedReturnTypeCoercion
     */
    public function byId(string $skillId, string $levelId): array
    {
        $qb = new QueryBuilder($this->em->getConnection());
        /** @var false|string $levelModeratorList */
        $levelModeratorList = $qb->from('level', 'l')
            ->select('l.moderator_list')
            ->where('l.id = :levelId AND l.skill_id = :skillId AND l.is_removed = false')
            ->setParameter('levelId', $levelId)
            ->setParameter('skillId', $skillId)
            ->setMaxResults(1)
            ->fetchOne();
        if (!$levelModeratorList) {
            throw new LevelNotFoundException(new LevelId($levelId));
        }

        $qb = new QueryBuilder($this->em->getConnection());
        $qb->from('member', 'm')
            ->select('m.id, m.name')
            ->where('m.id::text = ANY (:moderator_list)')
            ->setParameter('moderator_list', $levelModeratorList);

        $result = $qb->executeQuery();

        return array_map($this->getFnMapping(), $result->fetchAllAssociative());
    }

    protected function getFnMapping(): \Closure
    {
        return function (array $data): MemberReadModel {
            return new MemberReadModel(
                (string) ($data['id'] ?? ''),
                (string) ($data['name'] ?? ''),
            );
        };
    }
}
