<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Level\InMemory;

use App\Context\Content\Application\Query\Level\ReadModel\LevelNotFoundBySlugException;
use App\Context\Content\Application\Query\Level\ReadModel\LevelReadModel;
use App\Context\Content\Application\Query\Level\ReadModel\LevelReadRepository;
use App\Context\Content\Domain\Model\Level\Exception\LevelNotFoundException;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelSlug;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryObjectStorage;

class LevelReadRepositoryInMemory implements LevelReadRepository
{
    /** @var InMemoryObjectStorage<LevelReadModel> */
    private InMemoryObjectStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryObjectStorage(LevelReadModel::class);
    }

    public function byId(string $skillId, string $levelId): LevelReadModel
    {
        $level = $this->storage->get($levelId);
        if (is_null($level)) {
            throw new LevelNotFoundException(new LevelId($levelId));
        }

        return $level;
    }

    public function getList(?Criteria $criteria = null): array
    {
        return $this->storage->all();
    }

    public function add(LevelReadModel $model): void
    {
        $this->storage->set($model->id, $model);
    }

    public function bySlug(string $skillSlug, string $levelSlug): LevelReadModel
    {
        $levels = array_combine(array_column($this->storage->all(), 'slug'), $this->storage->all());
        if (!array_key_exists($levelSlug, $levels)) {
            throw new LevelNotFoundBySlugException(new LevelSlug($levelSlug));
        }

        return $levels[$levelSlug];
    }
}
