<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Member\Doctrine;

use App\Context\Content\Application\Query\Member\ReadModel\MemberReadModel;
use App\Context\Content\Application\Query\Member\ReadModel\MemberReadRepository;
use App\Context\Content\Domain\Model\Member\Exception\MemberNotFoundException;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Domain\Criteria\Field;
use App\Context\Shared\Domain\Criteria\Filter;
use App\Context\Shared\Domain\Criteria\FilterOperator;
use App\Context\Shared\Domain\Criteria\FilterStringValue;
use App\Context\Shared\Infrastructure\Doctrine\Criteria\CriteriaConvert;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;

class MemberReadModelRepositoryDoctrine implements MemberReadRepository
{
    public function __construct(
        private EntityManagerInterface $em
    ) {
    }

    public function byId(string $memberId): MemberReadModel
    {
        $criteria = new Criteria();
        $criteria->filters()
            ->add(
                new Filter(
                    new Field('id'),
                    FilterOperator::EQ,
                    new FilterStringValue($memberId)
                )
            );
        $criteria->setLimit(1);

        $member = current($this->search($criteria));
        if ($member) {
            return $member;
        }

        throw new MemberNotFoundException(new MemberId($memberId));
    }

    /**
     * @return MemberReadModel[]
     *
     * @throws \Doctrine\DBAL\Exception
     *
     * @psalm-suppress MixedReturnTypeCoercion
     */
    public function search(?Criteria $criteria = null): array
    {
        $qb = new QueryBuilder($this->em->getConnection());
        $qb->from('member', 'm')
            ->select('m.id, m.name');

        if ($criteria) {
            $qb = CriteriaConvert::criteriaToQueueBuilder($qb, $criteria);
        }

        $result = $qb->executeQuery();

        return array_map($this->getFnMapping(), $result->fetchAllAssociative());
    }

    protected function getFnMapping(): \Closure
    {
        return function (array $data): MemberReadModel {
            return new MemberReadModel(
                (string) ($data['id'] ?? ''),
                (string) ($data['name'] ?? ''),
            );
        };
    }
}
