<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Member\InMemory;

use App\Context\Content\Application\Query\Member\ReadModel\MemberReadModel;
use App\Context\Content\Application\Query\Member\ReadModel\MemberReadRepository;
use App\Context\Content\Domain\Model\Member\Exception\MemberNotFoundException;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryObjectStorage;

class MemberReadRepositoryInMemory implements MemberReadRepository
{
    /** @var InMemoryObjectStorage<MemberReadModel> */
    private InMemoryObjectStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryObjectStorage(MemberReadModel::class);
    }

    public function byId(string $memberId): MemberReadModel
    {
        $member = $this->storage->get($memberId);
        if (is_null($member)) {
            throw new MemberNotFoundException(new MemberId($memberId));
        }

        return $member;
    }

    public function search(?Criteria $criteria = null): array
    {
        return $this->storage->all();
    }

    public function add(MemberReadModel $model): void
    {
        $this->storage->set($model->memberId, $model);
    }
}
