<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Question\InMemory;

use App\Context\Content\Application\Query\Question\ReadModel\QuestionNotFoundBySlugException;
use App\Context\Content\Application\Query\Question\ReadModel\QuestionReadModel;
use App\Context\Content\Application\Query\Question\ReadModel\QuestionReadRepository;
use App\Context\Content\Domain\Model\Question\Exception\QuestionNotFoundException;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionSlug;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryObjectStorage;

class QuestionReadRepositoryInMemory implements QuestionReadRepository
{
    /** @var InMemoryObjectStorage<QuestionReadModel> */
    private InMemoryObjectStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryObjectStorage(QuestionReadModel::class);
    }

    public function byId(string $skillId, string $levelId, string $questionId): QuestionReadModel
    {
        $question = $this->storage->get($questionId);
        if (is_null($question)) {
            throw new QuestionNotFoundException(new QuestionId($questionId));
        }

        return $question;
    }

    public function getList(?Criteria $criteria = null): array
    {
        return $this->storage->all();
    }

    public function add(QuestionReadModel $model): void
    {
        $this->storage->set($model->id, $model);
    }

    public function bySlug(string $skillSlug, string $levelSlug, string $questionSlug): QuestionReadModel
    {
        $questions = array_combine(array_column($this->storage->all(), 'slug'), $this->storage->all());
        if (!array_key_exists($questionSlug, $questions)) {
            throw new QuestionNotFoundBySlugException(new QuestionSlug($questionSlug));
        }

        return $questions[$questionSlug];
    }
}
