<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Skill\Doctrine;

use App\Context\Content\Application\Query\Member\ReadModel\MemberReadModel;
use App\Context\Content\Application\Query\Skill\ReadModel\SkillModeratorsReadRepository;
use App\Context\Content\Domain\Model\Skill\Exception\SkillNotFoundException;
use App\Context\Content\Domain\Model\Skill\SkillId;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;

class SkillModeratorsReadModelRepositoryDoctrine implements SkillModeratorsReadRepository
{
    public function __construct(
        private readonly EntityManagerInterface $em,
    ) {
    }

    /**
     * @return list<MemberReadModel>
     *
     * @throws \Doctrine\DBAL\Exception
     *
     * @psalm-suppress MixedReturnTypeCoercion
     */
    public function byId(string $skillId): array
    {
        $qb = new QueryBuilder($this->em->getConnection());
        /** @var false|string $skillModeratorList */
        $skillModeratorList = $qb->from('skill', 's')
            ->select('s.moderator_list')
            ->where('s.id = :skillId AND s.is_removed = false')
            ->setParameter('skillId', $skillId)
            ->setMaxResults(1)
            ->fetchOne();
        if (!$skillModeratorList) {
            throw new SkillNotFoundException(new SkillId($skillId));
        }

        $qb = new QueryBuilder($this->em->getConnection());
        $qb->from('member', 'm')
            ->select('m.id, m.name')
            ->where('m.id::text = ANY (:moderator_list)')
            ->setParameter('moderator_list', $skillModeratorList);

        $result = $qb->executeQuery();

        return array_map($this->getFnMapping(), $result->fetchAllAssociative());
    }

    protected function getFnMapping(): \Closure
    {
        return function (array $data): MemberReadModel {
            return new MemberReadModel(
                (string) ($data['id'] ?? ''),
                (string) ($data['name'] ?? ''),
            );
        };
    }
}
