<?php

namespace App\Context\Content\Infrastructure\Repository\Application\Query\Skill\InMemory;

use App\Context\Content\Application\Query\Skill\ReadModel\SkillNotFoundBySlugException;
use App\Context\Content\Application\Query\Skill\ReadModel\SkillReadModel;
use App\Context\Content\Application\Query\Skill\ReadModel\SkillReadRepository;
use App\Context\Content\Domain\Model\Skill\Exception\SkillNotFoundException;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillSlug;
use App\Context\Shared\Domain\Criteria\Criteria;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryObjectStorage;

class SkillReadRepositoryInMemory implements SkillReadRepository
{
    /** @var InMemoryObjectStorage<SkillReadModel> */
    private InMemoryObjectStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryObjectStorage(SkillReadModel::class);
    }

    public function byId(string $id): SkillReadModel
    {
        $skill = $this->storage->get($id);
        if (is_null($skill)) {
            throw new SkillNotFoundException(new SkillId($id));
        }

        return $skill;
    }

    public function getList(?Criteria $criteria = null): array
    {
        return $this->storage->all();
    }

    public function add(SkillReadModel $model): void
    {
        $this->storage->set($model->id, $model);
    }

    public function bySlug(string $slug): SkillReadModel
    {
        $skills = array_combine(array_column($this->storage->all(), 'slug'), $this->storage->all());
        if (!array_key_exists($slug, $skills)) {
            throw new SkillNotFoundBySlugException(new SkillSlug($slug));
        }

        return $skills[$slug];
    }
}
