<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Level\Doctrine\Repository;

use App\Context\Content\Domain\Model\Level\Exception\LevelAlreadyRemoveException;
use App\Context\Content\Domain\Model\Level\Exception\LevelNotFoundException;
use App\Context\Content\Domain\Model\Level\Level;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Infrastructure\Repository\Domain\Skill\Doctrine\Repository\DoctrineSkillRepositoryPersistence;

class DoctrineLevelRepositoryPersistence implements LevelRepositoryPersistence, LevelRepositoryReader
{
    public function __construct(
        private readonly DoctrineSkillRepositoryPersistence $doctrineSkillRepository,
        private readonly DoctrineLevelRepository $doctrineLevelRepository
    ) {
    }

    public function persist(Level $level): void
    {
        $this->doctrineLevelRepository->save($level);
    }

    public function remove(LevelId $id): void
    {
        // TODO: Implement remove() method.
    }

    public function findById(LevelId $id): Level
    {
        $level = $this->doctrineLevelRepository->find($id);
        if (!$level) {
            throw new LevelNotFoundException($id);
        }

        return $level;
    }

    public function getActiveById(SkillId $skillId, LevelId $levelId): Level
    {
        $this->doctrineSkillRepository->getActiveById($skillId);

        $level = $this->findById($levelId);
        if ($level->isRemoved()->value()) {
            throw new LevelAlreadyRemoveException($levelId);
        }

        return $level;
    }
}
