<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Level\InMemory;

use App\Context\Content\Domain\Model\Level\Exception\LevelAlreadyRemoveException;
use App\Context\Content\Domain\Model\Level\Exception\LevelNotFoundException;
use App\Context\Content\Domain\Model\Level\Level;
use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Level\LevelRepositoryPersistence;
use App\Context\Content\Domain\Model\Level\LevelRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryAggregateStorage;

class LevelRepositoryInMemory implements LevelRepositoryPersistence, LevelRepositoryReader
{
    /**
     * @var InMemoryAggregateStorage<Level>
     */
    private InMemoryAggregateStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryAggregateStorage(Level::class);
    }

    public function persist(Level $level): void
    {
        $this->storage->set($level);
    }

    public function remove(LevelId $id): void
    {
        $this->storage->remove($id);
    }

    public function findById(LevelId $id): Level
    {
        $level = $this->storage->get($id);
        if (is_null($level)) {
            throw new LevelNotFoundException($id);
        }

        return $level;
    }

    public function getActiveById(SkillId $skillId, LevelId $levelId): Level
    {
        $level = $this->findById($levelId);
        if ($level->isRemoved()->value()) {
            throw new LevelAlreadyRemoveException($levelId);
        }

        return $level;
    }
}
