<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Member\Doctrine\Repository;

use App\Context\Content\Domain\Model\Member\Exception\MemberNotFoundException;
use App\Context\Content\Domain\Model\Member\Member;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Content\Domain\Model\Member\MemberRepositoryPersistence;
use App\Context\Content\Domain\Model\Member\MemberRepositoryReader;

class DoctrineMemberRepositoryPersistence implements MemberRepositoryPersistence, MemberRepositoryReader
{
    public function __construct(
        private readonly DoctrineMemberRepository $doctrineMemberRepository
    ) {
    }

    public function persist(Member $member): void
    {
        $this->doctrineMemberRepository->save($member);
    }

    public function remove(MemberId $memberId): void
    {
        $member = $this->findById($memberId);
        $this->doctrineMemberRepository->delete($member);
    }

    public function findById(MemberId $memberId): Member
    {
        $member = $this->doctrineMemberRepository->find($memberId);
        if (!$member) {
            throw new MemberNotFoundException($memberId);
        }

        return $member;
    }
}
