<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Member\InMemory;

use App\Context\Content\Domain\Model\Member\Exception\MemberNotFoundException;
use App\Context\Content\Domain\Model\Member\Member;
use App\Context\Content\Domain\Model\Member\MemberId;
use App\Context\Content\Domain\Model\Member\MemberRepositoryPersistence;
use App\Context\Content\Domain\Model\Member\MemberRepositoryReader;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryAggregateStorage;

class MemberRepositoryInMemory implements MemberRepositoryPersistence, MemberRepositoryReader
{
    /**
     * @var InMemoryAggregateStorage<Member>
     */
    private InMemoryAggregateStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryAggregateStorage(Member::class);
    }

    public function persist(Member $member): void
    {
        $this->storage->set($member);
    }

    public function remove(MemberId $memberId): void
    {
        $this->storage->remove($memberId);
    }

    public function findById(MemberId $memberId): Member
    {
        $member = $this->storage->get($memberId);
        if (is_null($member)) {
            throw new MemberNotFoundException($memberId);
        }

        return $member;
    }
}
