<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Question\Doctrine\Repository;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Exception\QuestionAlreadyRemoveException;
use App\Context\Content\Domain\Model\Question\Exception\QuestionNotFoundException;
use App\Context\Content\Domain\Model\Question\Question;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Infrastructure\Repository\Domain\Level\Doctrine\Repository\DoctrineLevelRepositoryPersistence;

class DoctrineQuestionRepositoryPersistence implements QuestionRepositoryPersistence, QuestionRepositoryReader
{
    public function __construct(
        private readonly DoctrineLevelRepositoryPersistence $doctrineLevelRepository,
        private readonly DoctrineQuestionRepository $doctrineQuestionRepository
    ) {
    }

    public function persist(Question $question): void
    {
        $this->doctrineQuestionRepository->save($question);
    }

    public function remove(QuestionId $id): void
    {
        // TODO: Implement remove() method.
    }

    public function findById(QuestionId $id): Question
    {
        $question = $this->doctrineQuestionRepository->find($id);
        if (!$question) {
            throw new QuestionNotFoundException($id);
        }

        return $question;
    }

    public function getActiveById(SkillId $skillId, LevelId $levelId, QuestionId $questionId): Question
    {
        $this->doctrineLevelRepository->getActiveById($skillId, $levelId);

        $question = $this->findById($questionId);
        if ($question->isRemoved()->value()) {
            throw new QuestionAlreadyRemoveException($questionId);
        }

        return $question;
    }
}
