<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Question\InMemory;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\Exception\QuestionAlreadyRemoveException;
use App\Context\Content\Domain\Model\Question\Exception\QuestionNotFoundException;
use App\Context\Content\Domain\Model\Question\Question;
use App\Context\Content\Domain\Model\Question\QuestionId;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryPersistence;
use App\Context\Content\Domain\Model\Question\QuestionRepositoryReader;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryAggregateStorage;

class QuestionRepositoryInMemory implements QuestionRepositoryPersistence, QuestionRepositoryReader
{
    /**
     * @var InMemoryAggregateStorage<Question>
     */
    private InMemoryAggregateStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryAggregateStorage(Question::class);
    }

    public function persist(Question $question): void
    {
        $this->storage->set($question);
    }

    public function remove(QuestionId $id): void
    {
        $this->storage->remove($id);
    }

    public function findById(QuestionId $id): Question
    {
        $question = $this->storage->get($id);
        if (is_null($question)) {
            throw new QuestionNotFoundException($id);
        }

        return $question;
    }

    public function getActiveById(SkillId $skillId, LevelId $levelId, QuestionId $questionId): Question
    {
        $question = $this->findById($questionId);
        if ($question->isRemoved()->value()) {
            throw new QuestionAlreadyRemoveException($questionId);
        }

        return $question;
    }
}
