<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Repository\Domain\Skill\Doctrine\Repository;

use App\Context\Content\Domain\Model\Skill\Exception\SkillAlreadyRemoveException;
use App\Context\Content\Domain\Model\Skill\Exception\SkillNotFoundException;
use App\Context\Content\Domain\Model\Skill\Skill;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;

class DoctrineSkillRepositoryPersistence implements SkillRepositoryPersistence, SkillRepositoryReader
{
    public function __construct(
        private readonly DoctrineSkillRepository $doctrineSkillRepository
    ) {
    }

    public function persist(Skill $skill): void
    {
        $this->doctrineSkillRepository->save($skill);
    }

    public function remove(SkillId $id): void
    {
        // TODO: Implement remove() method.
    }

    public function findById(SkillId $id): Skill
    {
        $skill = $this->doctrineSkillRepository->find($id);
        if (!$skill) {
            throw new SkillNotFoundException($id);
        }

        return $skill;
    }

    public function getActiveById(SkillId $skillId): Skill
    {
        $skill = $this->findById($skillId);
        if ($skill->isRemoved()->value()) {
            throw new SkillAlreadyRemoveException($skillId);
        }

        return $skill;
    }
}
