<?php

namespace App\Context\Content\Infrastructure\Repository\Domain\Skill\InMemory;

use App\Context\Content\Domain\Model\Skill\Exception\SkillAlreadyRemoveException;
use App\Context\Content\Domain\Model\Skill\Exception\SkillNotFoundException;
use App\Context\Content\Domain\Model\Skill\Skill;
use App\Context\Content\Domain\Model\Skill\SkillId;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryPersistence;
use App\Context\Content\Domain\Model\Skill\SkillRepositoryReader;
use App\Context\Shared\Infrastructure\Persistence\InMemory\InMemoryAggregateStorage;

class SkillRepositoryInMemory implements SkillRepositoryPersistence, SkillRepositoryReader
{
    /**
     * @var InMemoryAggregateStorage<Skill>
     */
    private InMemoryAggregateStorage $storage;

    public function __construct()
    {
        $this->storage = new InMemoryAggregateStorage(Skill::class);
    }

    public function persist(Skill $skill): void
    {
        $this->storage->set($skill);
    }

    public function remove(SkillId $id): void
    {
        $this->storage->remove($id);
    }

    public function findById(SkillId $id): Skill
    {
        $skill = $this->storage->get($id);
        if (is_null($skill)) {
            throw new SkillNotFoundException($id);
        }

        return $skill;
    }

    public function getActiveById(SkillId $skillId): Skill
    {
        $skill = $this->findById($skillId);
        if ($skill->isRemoved()->value()) {
            throw new SkillAlreadyRemoveException($skillId);
        }

        return $skill;
    }
}
