<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Service\Git;

use App\Context\Content\Application\Service\Git\GitStore;
use Git\Exception\GitCloneException;
use Git\Exception\GitPullException;
use Git\Exception\NotGitFolderException;
use Git\Git;

class GitRemoteStore implements GitStore
{
    public function __construct(
        protected readonly string $folder,
        protected readonly string $remote,
    ) {
    }

    /**
     * @throws GitCloneException
     */
    public function pull(): void
    {
        Git::clone($this->folder, $this->remote);
    }

    /**
     * @throws GitPullException
     * @throws NotGitFolderException
     */
    public function push(): void
    {
        $git = new Git($this->folder);

        $git->add('.');
        $git->commit('Auto-export content from system');

        $git->push();
    }

    public function clean(): void
    {
        try {
            $git = new Git($this->folder);
            $git->rm();
        } catch (NotGitFolderException) {}
    }
}
