<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Specification\Level;

use App\Context\Content\Domain\Model\Level\LevelSlug;
use App\Context\Content\Domain\Model\Level\Specification\LevelSlugUniqueSpec as ILevelSlugUniqueSpec;
use App\Context\Content\Domain\Model\Skill\SkillId;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;

class LevelSlugUniqueSpec implements ILevelSlugUniqueSpec
{
    public function __construct(
        private readonly EntityManagerInterface $em
    ) {
    }

    public function isSatisfiedBy(SkillId $skillId, LevelSlug $levelSlug): bool
    {
        $qb = new QueryBuilder($this->em->getConnection());
        /** @var false|string $level */
        $level = $qb->from('level', 'l')
            ->select('l.id')
            ->where('l.slug = :slug AND l.skill_id = :skillId')
            ->setParameter('slug', $levelSlug->value())
            ->setParameter('skillId', $skillId->value())
            ->setMaxResults(1)
            ->fetchOne();

        if (empty($level)) {
            return true;
        }

        return false;
    }
}
