<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Specification\Question;

use App\Context\Content\Domain\Model\Level\LevelId;
use App\Context\Content\Domain\Model\Question\QuestionSlug;
use App\Context\Content\Domain\Model\Question\Specification\QuestionSlugUniqueSpec as IQuestionSlugUniqueSpec;
use Doctrine\DBAL\Query\QueryBuilder;
use Doctrine\ORM\EntityManagerInterface;

class QuestionSlugUniqueSpec implements IQuestionSlugUniqueSpec
{
    public function __construct(
        private readonly EntityManagerInterface $em
    ) {
    }

    public function isSatisfiedBy(LevelId $levelId, QuestionSlug $questionSlug): bool
    {
        $qb = new QueryBuilder($this->em->getConnection());
        /** @var false|string $question */
        $question = $qb->from('question', 'q')
            ->select('q.id')
            ->where('q.slug = :slug AND q.level_id = :level_id')
            ->setParameter('slug', (string) $questionSlug)
            ->setParameter('level_id', (string) $levelId)
            ->setMaxResults(1)
            ->fetchOne();

        if (empty($question)) {
            return true;
        }

        return false;
    }
}
