<?php

declare(strict_types=1);

namespace App\Context\Content\Infrastructure\Subscriber\Member;

use App\Context\Auth\Domain\Model\User\Event\UserRegistered;
use App\Context\Content\Application\Command\Member\RegisterMember\RegisterMemberCommand;
use App\Context\Shared\Domain\Bus\Command\CommandBus;
use App\Context\Shared\Domain\Bus\Command\CommandResponse;
use App\Context\Shared\Domain\Bus\Event\Subscriber;

class ContentMemberRegisterSubscriber implements Subscriber
{
    public function __construct(
        public CommandBus $commandBus,
    ) {
    }

    public function subscribeTo(): array
    {
        return [
            UserRegistered::class,
        ];
    }

    public function __invoke(UserRegistered $event): CommandResponse
    {
        return $this->commandBus->dispatch(
            new RegisterMemberCommand(
                $event->getAggregateId(),
                $event->firstname.' '.$event->lastname,
            )
        );
    }
}
